# Fast Marching Method

![Jerboa_FMM](2rugs_fmm_fmmha.png)

## Introduction
The main goal of this implementation is to be able to use the topology to apply
the Fast Marching Method to calculate distances and simulate front propagation. 

The implementation of this algorithm mainly relies on two sources :
1. The work done and explained mainly by James Sethian. Many of his articles can be found on his website https://math.berkeley.edu/~sethian/2006/Explanations/fast_marching_explain.html
2. _J. A. Barentzen, “On the Implementation of Fast Marching Methods for 3D Lattices”, Tech. report, Tech. Univ. of Denmark, 2001._ 
    An article describing how to implement the fast marching method on 3D lattices. 

To do so, we considered each orbit `<1,2,3>` as a voxel of the lattice.
We also add an embedding corresponding to a min priority queue to store efficiently
the narrow_band (cf. 2) values.

## Embeddings

- **point** : the coords (x,y,z) of one `vertex <1,2,3>`.
- **fmmVoxel** : a `vertex <1,2,3>` voxel in the lattice. Properties : 
  - coord : its coordinates in 3D.
  - state : an enum {FROZEN, NARROW_BAND, FAR} (cf. 2) (_FAR_ by default)
  - speed : the speed of the front on this voxel (_1_ by default)
  - distance : the distance from the starting point (_infinite_ by default)
  - isInit : is the voxel a starting point ("false" by default)
  - dart : the jerboa dart linked to the voxel
  - mins : number of axe's array (size=3 in 3D) containing the min distance value of it's frozen neighbours. Used to calculate
  the coefficients (_coeff_ attribute).
  - minsHA : number of axe's array (size=3 in 3D) containing the min distance value of it's two-steps frozen neighbours. Used to calculate
  the coefficients (_coeff_ attribute).
  - coeff : array of 3 values used to solve the estimation of the Eikonal equation (cf. 1 & 2)
- **fmmPriorityQueue** : the min priority queue shared by all the connected component (composante connexe) `<0,1,2,3>`
                         to store efficiently the narrow_band voxels. (cf. 2)
  - fmmQueue : the fmmVoxel priority queue
  - max : the priority queue's higher distance. Used in the **color** embedding 
- **color** : the color linked to a `vertex <1,2,3>` orbit. Used to make the color map.

## Rules
### Classic fast marching method
- **FMMInit** : in the connected component, for each `vertex <1,2,3>`, create a fmmVoxel with the right
                coordinates and default values. (see fmmVoxel embedding). Initialize also the fmmPriorityQueue for the whole
               the connected component.
<br></br>
- **FMMSetStart** : set the selected fmmVoxel `vertex <1,2,3>` as a starting point (state=FROZEN, distance=0 and 
isInit=true). We can chose several points
<br></br>
- **FMMGetMinsFromNgbrs** : for a fmmVoxel `vertex <1,2,3>`, set its _mins_ values from its frozen neighbours' minimum distance value 
for each axe (here in 3D, so 3 axes).
<br></br>
- **FMMComputeCoeffsAndDistance** : for a fmmVoxel `vertex <1,2,3>`, compute the estimation of the Eikonal equation from 
the coeffs calculated thanks to its mins values.
<br></br>
- **FMMColorVertex** : color a `vertex <1,2,3>` according to its distance value. From red (distance=0) to 
purple (distance=1).

### Fast marching high accuracy (FMMHA)

A set of rules have been especially made for this fast marching adaption (cf. 2 for details on the FMMHA). Theses rules 
are suffixed by "HA" and replaces the classic rules when there is one. The other rules stay unchanged.

- **FMMGetMinsFromNgbrsHA** : for a fmmVoxel `vertex <1,2,3>`, set its _mins_ values from its **two-step** frozen neighbours' minimum 
distance value for each axe (here in 3D, so 3 axes). The two-steps values are stored in the _minHA_ fmmVoxel's value.  

- **FMMComputeCoeffsAndDistanceHA** : for a fmmVoxel `vertex <1,2,3>`, compute the estimation of the Eikonal equation from
  the coeffs calculated thanks to its mins and minsHA values. If the minHA values can not be used, we use the mins values
  (i.e. if we can not use the two-step neighbours in the calculus, then we use the one-step neighbours)

## Script

- **FMMComputeDistance** : for a fmmVoxel `vertex <1,2,3>`, compute its distance by applying the rules 
_FMMGetMinsFromNgbrs_ (FMMGetMinsFromNgbrsHA if we use FMMHA) and _FMMComputeCoeffsAndDistance_ (FMMComputeCoeffsAndDistanceHA). 
Correspond to the function "recompute(..)" in the Barentzen article (cf. 2). 
<br></br>
- **FMMGoWithoutInit** : for correctly initialized FMM connected components, compute the neighbours of the starting 
points in a first loop. Then in a second loop, perform the fast marching algorithm to estimate the Eikonal equation on all the fmmVoxels. 
Finally, apply _FMMColorVertex_ to color the connected components according to the distances computed.
<br></br>
- **FMMGo** : For a connected component, apply the Fast Marching Method to compute the front popragation from one or 
several start points. Applies  _FMMInit_ and, for each starting point, _FMMSetStart_ to correctly 
initialize the component. Then apply _FMMGoWithoutInit_ to perform the FMM algorithm. 
