# Fast Marching Method on 3D triangulated acute meshes

![Jerboa_FMM](fmm_david_2_sources.png)

## Introduction
The main goal of this implementation is to be able to use the topology to apply
the Fast Marching Method to calculate distances and simulate front propagation. 
More precisely on a Fast Marching method implementation on 3D triangulated acute meshes.

The implementation of this algorithm mainly relies on two sources :
1. _Kimmel, Ron & Sethian, JA. (1998). Computing Geodesic Paths on Manifolds. Proceedings of the National Academy of Sciences of the United States of America. 95. 8431-5. 10.1073/pnas.95.15.8431._ 
    An article describing how to extend the fast marching method on 3D triangulated meshes. 
2. _Gabriel Peyre (2022). Toolbox Fast Marching (https://www.mathworks.com/matlabcentral/fileexchange/6110-toolbox-fast-marching), MATLAB Central File Exchange. Retrieved April 25, 2022._

To do so, we considered each orbit `<1,2,3>` as a voxel of the triangulated mesh, 
each orbit `<0,1,3>` as a triangle face.
We added an embedding corresponding to a min priority queue to store efficiently
the narrow_band values (see the fast marching implementation on lattices for further details).

## Embeddings

- **point** : the coords (x,y,z) of one `vertex <1,2,3>`.
- **fmmVoxel** : a `vertex <1,2,3>` voxel in the lattice. Properties : 
  - id : unique identifier.
  - coord : its coordinates in 3D.
  - state : an enum {FROZEN, NARROW_BAND, FAR} (_FAR_ by default)
  - speed : the speed of the front on this voxel (_1_ by default)
  - distance : the distance from the starting point (_infinite_ by default)
  - isInit : is the voxel a starting point ("false" by default)
  - dart : the jerboa dart linked to the voxel
  - mins : number of axe's array (size=3 in 3D) containing the min distance value of it's frozen neighbours. Used to calculate
  the coefficients (_coeff_ attribute).
  - minsGridSpace : array containing the gridspace value between an fmmVoxel and the min of its neighbours on each axe. Used to calculate 
  the estimation of the eikonal equation.
  - coeff : array of 3 values used to solve the estimation of the Eikonal equation.
  - controlDistance : a distance value used to compare with the calculated one (_distance_). 
- **fmmPriorityQueue** : the min priority queue shared by all the connected component (composante connexe) `<0,1,2,3>`
                         to store efficiently the narrow_band voxels. (cf. 2)
  - fmmQueue : the fmmVoxel priority queue
  - max : the priority queue's higher distance. Used in the **color** embedding 
  - maxControlDistance : the higher control distance. Used to create a distance map.
- **color** : the color linked to a `vertex <1,2,3>` orbit. Used to make the color map.

## Rules

- **FMMInit** : in the connected component, for each `vertex <1,2,3>`, create a fmmVoxel with the right
                coordinates and default values. (see fmmVoxel embedding). Initialize also the fmmPriorityQueue for the whole
               the connected component.
<br></br>
- **FMMSetStart** : set the selected fmmVoxel `vertex <1,2,3>` as a starting point (state=FROZEN, distance=0 and 
isInit=true).
<br></br>
- **FMMMeshComputeFace** : compute the front arrival time on the selected fmmVoxel `<1,2,3>` for one triangle face.
<br></br>
- **FMMMeshComputeFaceGP** : compute the front arrival time on the selected fmmVoxel `<1,2,3>` for one triangle face 
using Gabriel Peyré's algorithm (cf. 2).
<br></br>
- **FMMColorVertex** : color a `vertex <1,2,3>` according to its distance value. From red (distance=0) to 
purple (distance=1).

## Scripts

- **FMMMeshUmbrella** : for a fmmVoxel `<1,2,3>`, compute its distance for each face linked to it using the rule 
_FMMMeshComputeFace_ or _FMMMeshComputeFaceGP_ and select the minimum computed distance. 
<br></br>
- **FMMMeshGoWithoutInit** : for an already initialised connected component, apply the Fast Marching Method to compute a
front propagation from the sources using _FMMMeshUmbrella_. First from the direct sources in the fist loop, then while
the priority queue is not empty.
<br><br>
- **FMMMeshMain** : For a connected component, apply the Fast Marching Method to compute the front propagation from sources. 
Applies _FMMInit_ and _FMMSetStart_ to initialise the Fast Marching then call _FMMMeshGoWithoutInit_ to perform the
front propagation. Finally, apply _FMMColorVertex_ to color the connected component according to the distances computed.
Several options when used : 
  - You can enter a list of targets' id (FMMVoxel.id), once the propagation has reached all of them it stops (see Fig.1)
  - You can use only FROZEN vertices while computing the gradients or using also the NARROW_BAND values (default behaviour)
  - In G.Peyré toolbox (see sources), he used a different way to compute arrival time, we tried to implement it. However this feature is not stable and we highly recommand not to use it for now.
<figure>
<img src="voxel_id.png" alt="drawing" style="width:40%">
  <figcaption>Fig.1 - Select targets FMMVoxel's id. It is usefull if you want the propagation to reach a specific point on the mesh.</figcaption>
</figure>
<figure>
<img src="frozen_voxels.png" alt="drawing" style="width:40%">
  <figcaption>Fig.2 - Choose only fozen vertex during computationy </figcaption>
</figure> 
<figure>
<img src="GP.png" alt="drawing" style="width:40%">
  <figcaption>Fig.2 - Choose G.Peyré computation (not stable in our implementation)</figcaption>
</figure> 